package io.scanbot.sdk.flutter.camera.usecase

import android.app.Application
import android.content.Context
import android.preference.PreferenceManager
import io.scanbot.sdk.PreferencesConstants
import io.scanbot.sdk.util.FileUtils
import io.scanbot.sdk.util.log.LoggerProvider
import java.io.File
import java.io.IOException
import java.lang.Exception

/**
 * File storage for mc images
 */
class MedicalCertificateImageStorage
constructor(val application: Context) {

    private val sdkMcImagesFolderName: String = "medical_certificates_images"

    /**
     * Provides directory which store all mc images
     *
     * @throws IOException
     * @return [File] directory, which will contain images for scanned mcs
     */
    @Throws(IOException::class)
    fun getMedicalCertificateImagesDirectory(): File {
        val sdkFilesDirPath = PreferenceManager.getDefaultSharedPreferences(application).getString(
            PreferencesConstants.CUSTOM_SDK_FILES_PATH, "")
        return if (io.scanbot.sdk.util.FileUtils.ensureDirExists(sdkFilesDirPath)) {
            FileUtils.getFilesDirOrShowError(sdkFilesDirPath, sdkMcImagesFolderName)
        } else {
            FileUtils.getFilesDirOrShowError(
                FileUtils.getAppFilesDirOrShowError(
                    application,
                    FileUtils.SCANBOT_SDK_FILES_DIR
                ).path, sdkMcImagesFolderName
            )
        }
    }

    /**
     *
     * Cleans a directory with all stored mc images stored
     */
    fun cleanupMedicalCertificateImagesDirectory() {
        try {
            getMedicalCertificateImagesDirectory().listFiles().forEach {
                it.delete()
            }
        } catch (e: Exception) {
            LoggerProvider.logger.logException(e)
        }
    }
}
