//
//  RTUUIPresenter.swift
//  scanbot_sdk
//
//  Created by Yevgeniy Knizhnik on 15.06.21.
//

import Foundation
import Flutter
import ScanbotSDK

class RTUUIPresenter: CallErrorHandler {
    
    private lazy var documentScannerPresenter = DocumentScannerPresenter()
    func showDocumentScanner(_ parameters: CallUIParameters) {
      
        guard let arguments = parameters.call.arguments as? [String: Any] else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        guard let config = SBSDKUIDocumentScannerConfiguration(json: arguments.encodedString ?? "" )  else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        
        if let presenter = parameters.presenter {
            self.documentScannerPresenter.present(on: presenter, with: config, result: parameters.result)
        }
    }
    
    private lazy var barcodeScannerPresenter = BarcodeScannerPresenter()
    func showBarcodeScanner(_ parameters: CallUIParameters) {
      
        guard let arguments = parameters.call.arguments as? [String: Any] else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        guard let config = SBSDKUIBarcodeScannerConfiguration(json: arguments.encodedString ?? "" )  else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }

        let codes = SBSDKBarcodeType.fromArguments(arguments)
        let documentTypes = SBSDKBarcodeDocumentType.fromArguments(arguments)
        
        guard let presenter = parameters.presenter else {
            return
        }
        
        self.barcodeScannerPresenter.present(on: presenter,
                                             with: config,
                                             machineCodeTypes: codes,
                                             barcodeDocumentTypes: documentTypes,
                                             result: parameters.result)
    }
    
    private lazy var mrzScannerPresenter = MRZScannerPresenter()
    func showMRZScanner(_ parameters: CallUIParameters) {
        
        guard let arguments = parameters.call.arguments as? [String: Any] else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        guard let config = SBSDKUIMRZScannerConfiguration(json: arguments.encodedString ?? "") else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        
        if let presenter = parameters.presenter {
            self.mrzScannerPresenter.present(on: presenter, with: config, result: parameters.result)
        }
    }
    
    private lazy var ehicScannerPresenter = EHICRecognizerPresenter()
    func showEHICScanner(_ parameters: CallUIParameters) {
        
        guard let arguments = parameters.call.arguments as? [String: Any] else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        guard let config = SBSDKUIHealthInsuranceCardScannerConfiguration(json: arguments.encodedString ?? "" )  else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        
        if let presenter = parameters.presenter {
            self.ehicScannerPresenter.present(on: presenter, with: config, result: parameters.result)
        }
    }
    
    private lazy var cropScreenPresenter = CropScreenPresenter()
    func showCropScreen(_ parameters: CallUIParameters) {
        
        guard let arguments = parameters.call.arguments as? [String: Any],
              let page = SBSDKUIPage(from: arguments) else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        let config = SBSDKUICroppingScreenConfiguration(from: arguments)
        
        if let presenter = parameters.presenter {
            self.cropScreenPresenter.present(on: presenter,
                                             with: page,
                                             configuration: config,
                                             result: parameters.result)
        }
    }
    
    private lazy var batchScannerPresenter = BatchScannerPresenter()
    func showBatchScanner(_ parameters: CallUIParameters, dataFormatterChannel: FlutterMethodChannel?) {
        
        guard let arguments = parameters.call.arguments as? [String: Any] else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        guard let config = SBSDKUIBarcodesBatchScannerConfiguration(json: arguments.encodedString ?? "" )  else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        let codes = SBSDKBarcodeType.fromArguments(arguments)
        let documentTypes = SBSDKBarcodeDocumentType.fromArguments(arguments)
        
        if let presenter = parameters.presenter {
            var dataMapper: SBSDKUIBarcodesBatchScannerMapping?
            if let shouldUseDataMapper = arguments["shouldUseBarcodeFormatter"] as? Bool, shouldUseDataMapper == true {
                dataMapper = BarcodeBatchDataMapper(dataChannel: dataFormatterChannel)
            }
            
            self.batchScannerPresenter.present(on: presenter,
                                               with: config,
                                               dataMapper: dataMapper,
                                               barcodeTypes: codes,
                                               barcodeDocumentTypes: documentTypes,
                                               result: parameters.result)
        }
    }
    
    private lazy var licensePlateScannerPresenter = LicensePlateScannerPresenter()
    func showLicensePlateScanner(_ parameters: CallUIParameters) {
        
        guard let arguments = parameters.call.arguments as? [String: Any] else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        guard let config = SBSDKUILicensePlateScannerConfiguration(json: arguments.encodedString ?? "" )  else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        
        if let presenter = parameters.presenter {
            self.licensePlateScannerPresenter.present(on: presenter, with: config, result: parameters.result)
        }
    }
    
    private lazy var genericDocumentRecognizerPresenter = GenericDocumentRecognizerPresenter()
    func showGenericDocumentRecognizer(_ parameters: CallUIParameters) {
        
        guard let arguments = parameters.call.arguments as? [String: Any] else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        guard let config = SBSDKUIGenericDocumentRecognizerConfiguration(json: arguments.encodedString ?? "" )  else {
            self.returnArgumentsError(call: parameters.call, result: parameters.result)
            return
        }
        
        if let presenter = parameters.presenter {
            
            self.genericDocumentRecognizerPresenter.present(on: presenter, with: config, result: parameters.result)
        }
    }
}
