import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:logging/logging.dart';

import '../barcode_scanning_data.dart';
import 'barcode_live_detection.dart';
import 'barcode_scanner_configuration.dart';
import 'classical_camera.dart';
export 'barcode_live_detection.dart';
export 'barcode_scanner_configuration.dart';
export 'classical_camera.dart';

class BarcodeScannerCamera extends StatelessWidget {
  const BarcodeScannerCamera(
      {Key? key,
      required this.cameraDetector,
      required this.configuration,
      required this.onWidgetReady,
      this.onCameraPreviewStarted,
      this.onHeavyOperationProcessing})
      : super(key: key);

  /// The detector that will be used to detect barcodes and return the result or error from it.
  final BarcodeCameraLiveDetector cameraDetector;

  /// The configuration of the camera and detection behavior.
  final BarcodeCameraConfiguration configuration;

  /// Runs when the camera is ready to be used. This happens not only once when the widget is created but also when the app returns to foreground and user pops to camera page from another page, if `ScanbotCamera.scanbotSdkRouteObserver` is registered
  final Function(ScanbotCameraController controller) onWidgetReady;

  /// Runs each time the camera preview is started. This differs for android and ios.
  /// In android  this function will be called after each successful snap while in ios only once.
  /// As ios not stopping camera preview after the snap
  final Function()? onCameraPreviewStarted;

  /// triggers when camera plugin make long work eg. saving snap to the image etc.
  final Function(bool)? onHeavyOperationProcessing;

  @override
  Widget build(BuildContext context) {
    var finder = configuration.finder;
    return ScanbotCamera(
      detector: cameraDetector,
      // Here we set initial configurations of the camera
      configuration: configuration,
      onMethodCall: _onMethodCall,
      onCameraReady: onWidgetReady,
      onCameraPreviewStarted: onCameraPreviewStarted,
      finder: finder,
      onHeavyOperationProcessing: onHeavyOperationProcessing,
    );
  }

  Future<dynamic> _onMethodCall(MethodCall methodCall) async {
    try {
      switch (methodCall.method) {
        case 'onBarcodeSelected':
          var barcodesJson = jsonDecode(methodCall.arguments);
          var barcodeItem = BarcodeItem.fromJson(barcodesJson);
          configuration.overlayConfiguration?.onBarcodeClicked
              ?.call(barcodeItem);
          break;
        default:
      }
    } catch (e) {
      Logger.root.warning(e);
    }
    return;
  }
}
