import '../sdk_utils.dart';
import 'common_generic_document.dart';

/// MRZ part of the document
class MRZ extends GenericDocumentWrapper {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "MRZ";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE = "MRZ";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  /// Unknown
  late TextFieldWrapper? unknown;

  /// Document number
  late TextFieldWrapper? documentNumber;

  /// Issuing authority
  late TextFieldWrapper? issuingAuthority;

  /// Office of issuance
  late ValidatedTextFieldWrapper? officeOfIssuance;

  /// Date of issuance
  late ValidatedTextFieldWrapper? dateOfIssuance;

  /// Given names
  late TextFieldWrapper givenNames;

  /// Surname
  late TextFieldWrapper surname;

  /// Nationality
  late TextFieldWrapper nationality;

  /// Birth date
  late TextFieldWrapper birthDate;

  /// Gender
  late TextFieldWrapper? gender;

  /// Expiry date
  late ValidatedTextFieldWrapper? expiryDate;

  /// Personal number
  late ValidatedTextFieldWrapper? personalNumber;

  /// Travel document type
  late TextFieldWrapper? travelDocType;

  /// Travel document type variant
  late TextFieldWrapper? travelDocTypeVariant;

  /// Optional 1
  late ValidatedTextFieldWrapper? optional1;

  /// Optional 2
  late ValidatedTextFieldWrapper? optional2;

  /// Document type code
  late TextFieldWrapper? documentTypeCode;

  /// PIN code
  late TextFieldWrapper? pinCode;

  /// Language code
  late TextFieldWrapper? languageCode;

  /// Version number
  late TextFieldWrapper? versionNumber;

  /// Check digit document number
  late ValidatedTextFieldWrapper? checkDigitDocumentNumber;

  /// Check digit birth date
  late ValidatedTextFieldWrapper? checkDigitBirthDate;

  /// Check digit expiry date
  late ValidatedTextFieldWrapper? checkDigitExpiryDate;

  /// Check digit personal number
  late ValidatedTextFieldWrapper? checkDigitPersonalNumber;

  /// Check  digit general
  late ValidatedTextFieldWrapper? checkDigitGeneral;

  MRZ(GenericDocument document) : super(document) {
    /// Unknown
    unknown = document
        .fieldByTypeName("Unknown")
        ?.let((field) => TextFieldWrapper(field));

    /// Document number
    documentNumber = document
        .fieldByTypeName("DocumentNumber")
        ?.let((field) => TextFieldWrapper(field));

    /// Issuing authority
    issuingAuthority = document
        .fieldByTypeName("IssuingAuthority")
        ?.let((field) => TextFieldWrapper(field));

    /// Office of issuance
    officeOfIssuance = document
        .fieldByTypeName("OfficeOfIssuance")
        ?.let((field) => ValidatedTextFieldWrapper(field));

    /// Date of issuance
    dateOfIssuance = document
        .fieldByTypeName("DateOfIssuance")
        ?.let((field) => ValidatedTextFieldWrapper(field));

    /// Given names
    givenNames = document
        .fieldByTypeName("GivenNames")!
        .let((field) => TextFieldWrapper(field));

    /// Surname
    surname = document
        .fieldByTypeName("Surname")!
        .let((field) => TextFieldWrapper(field));

    /// Nationality
    nationality = document
        .fieldByTypeName("Nationality")!
        .let((field) => TextFieldWrapper(field));

    /// Birth date
    birthDate = document
        .fieldByTypeName("BirthDate")!
        .let((field) => TextFieldWrapper(field));

    /// Gender
    gender = document
        .fieldByTypeName("Gender")
        ?.let((field) => TextFieldWrapper(field));

    /// Expiry date
    expiryDate = document
        .fieldByTypeName("ExpiryDate")
        ?.let((field) => ValidatedTextFieldWrapper(field));

    /// Personal number
    personalNumber = document
        .fieldByTypeName("PersonalNumber")
        ?.let((field) => ValidatedTextFieldWrapper(field));

    /// Travel document type
    travelDocType = document
        .fieldByTypeName("TravelDocType")
        ?.let((field) => TextFieldWrapper(field));

    /// Travel document type variant
    travelDocTypeVariant = document
        .fieldByTypeName("TravelDocTypeVariant")
        ?.let((field) => TextFieldWrapper(field));

    /// Optional 1
    optional1 = document
        .fieldByTypeName("Optional1")
        ?.let((field) => ValidatedTextFieldWrapper(field));

    /// Optional 2
    optional2 = document
        .fieldByTypeName("Optional2")
        ?.let((field) => ValidatedTextFieldWrapper(field));

    /// Document type code
    documentTypeCode = document
        .fieldByTypeName("DocumentTypeCode")
        ?.let((field) => TextFieldWrapper(field));

    /// PIN code
    pinCode = document
        .fieldByTypeName("PINCode")
        ?.let((field) => TextFieldWrapper(field));

    /// Language code
    languageCode = document
        .fieldByTypeName("LanguageCode")
        ?.let((field) => TextFieldWrapper(field));

    /// Version number
    versionNumber = document
        .fieldByTypeName("VersionNumber")
        ?.let((field) => TextFieldWrapper(field));

    /// Check digit document number
    checkDigitDocumentNumber = document
        .fieldByTypeName("CheckDigitDocumentNumber")
        ?.let((field) => ValidatedTextFieldWrapper(field));

    /// Check digit birth date
    checkDigitBirthDate = document
        .fieldByTypeName("CheckDigitBirthDate")
        ?.let((field) => ValidatedTextFieldWrapper(field));

    /// Check digit expiry date
    checkDigitExpiryDate = document
        .fieldByTypeName("CheckDigitExpiryDate")
        ?.let((field) => ValidatedTextFieldWrapper(field));

    /// Check digit personal number
    checkDigitPersonalNumber = document
        .fieldByTypeName("CheckDigitPersonalNumber")
        ?.let((field) => ValidatedTextFieldWrapper(field));

    /// Check  digit general
    checkDigitGeneral = document
        .fieldByTypeName("CheckDigitGeneral")
        ?.let((field) => ValidatedTextFieldWrapper(field));
  }
}

///  Class that contains all fields signatures of the MRZ document. Use it to Configure the document fields when running the scanner screen.
class MRZFieldsSignatures {
  static const String Unknown = MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "Unknown";
  static const String DocumentNumber =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "DocumentNumber";
  static const String IssuingAuthority =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "IssuingAuthority";
  static const String OfficeOfIssuance =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "OfficeOfIssuance";
  static const String DateOfIssuance =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "DateOfIssuance";
  static const String GivenNames =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "GivenNames";
  static const String Surname = MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "Surname";
  static const String Nationality =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "Nationality";
  static const String BirthDate =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "BirthDate";
  static const String Gender = MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "Gender";
  static const String ExpiryDate =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "ExpiryDate";
  static const String PersonalNumber =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "PersonalNumber";
  static const String TravelDocType =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "TravelDocType";
  static const String TravelDocTypeVariant =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "TravelDocTypeVariant";
  static const String Optional1 =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "Optional1";
  static const String Optional2 =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "Optional2";
  static const String DocumentTypeCode =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "DocumentTypeCode";
  static const String PINCode = MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "PINCode";
  static const String LanguageCode =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "LanguageCode";
  static const String VersionNumber =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "VersionNumber";
  static const String CheckDigitDocumentNumber =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "CheckDigitDocumentNumber";
  static const String CheckDigitBirthDate =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "CheckDigitBirthDate";
  static const String CheckDigitExpiryDate =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "CheckDigitExpiryDate";
  static const String CheckDigitPersonalNumber =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "CheckDigitPersonalNumber";
  static const String CheckDigitGeneral =
      MRZ.DOCUMENT_NORMALIZED_TYPE + "." + "CheckDigitGeneral";
  const MRZFieldsSignatures();
}

/// German ID card, front side
class DeIdCardFront extends GenericDocumentWrapper {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "DeIdCardFront";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE = "DeIdCardFront";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  /// Document ID number (in the top-right corner)
  late TextFieldWrapper id;

  /// Surname
  late TextFieldWrapper surname;

  /// Maiden name
  late TextFieldWrapper? maidenName;

  /// Given names
  late TextFieldWrapper givenNames;

  /// Birth date
  late ValidatedTextFieldWrapper birthDate;

  /// Nationality
  late TextFieldWrapper nationality;

  /// Birthplace
  late TextFieldWrapper birthplace;

  /// Expiry date
  late ValidatedTextFieldWrapper expiryDate;

  /// PIN
  late TextFieldWrapper pin;

  /// Photo image
  late TextFieldWrapper photo;

  /// Signature image
  late TextFieldWrapper signature;

  DeIdCardFront(GenericDocument document) : super(document) {
    /// Document ID number (in the top-right corner)
    id =
        document.fieldByTypeName("ID")!.let((field) => TextFieldWrapper(field));

    /// Surname
    surname = document
        .fieldByTypeName("Surname")!
        .let((field) => TextFieldWrapper(field));

    /// Maiden name
    maidenName = document
        .fieldByTypeName("MaidenName")
        ?.let((field) => TextFieldWrapper(field));

    /// Given names
    givenNames = document
        .fieldByTypeName("GivenNames")!
        .let((field) => TextFieldWrapper(field));

    /// Birth date
    birthDate = document
        .fieldByTypeName("BirthDate")!
        .let((field) => ValidatedTextFieldWrapper(field));

    /// Nationality
    nationality = document
        .fieldByTypeName("Nationality")!
        .let((field) => TextFieldWrapper(field));

    /// Birthplace
    birthplace = document
        .fieldByTypeName("Birthplace")!
        .let((field) => TextFieldWrapper(field));

    /// Expiry date
    expiryDate = document
        .fieldByTypeName("ExpiryDate")!
        .let((field) => ValidatedTextFieldWrapper(field));

    /// PIN
    pin = document
        .fieldByTypeName("PIN")!
        .let((field) => TextFieldWrapper(field));

    /// Photo image
    photo = document
        .fieldByTypeName("Photo")!
        .let((field) => TextFieldWrapper(field));

    /// Signature image
    signature = document
        .fieldByTypeName("Signature")!
        .let((field) => TextFieldWrapper(field));
  }
}

///  Class that contains all fields signatures of the DeIdCardFront document. Use it to Configure the document fields when running the scanner screen.
class DeIdCardFrontFieldsSignatures {
  static const String ID = DeIdCardFront.DOCUMENT_NORMALIZED_TYPE + "." + "ID";
  static const String Surname =
      DeIdCardFront.DOCUMENT_NORMALIZED_TYPE + "." + "Surname";
  static const String MaidenName =
      DeIdCardFront.DOCUMENT_NORMALIZED_TYPE + "." + "MaidenName";
  static const String GivenNames =
      DeIdCardFront.DOCUMENT_NORMALIZED_TYPE + "." + "GivenNames";
  static const String BirthDate =
      DeIdCardFront.DOCUMENT_NORMALIZED_TYPE + "." + "BirthDate";
  static const String Nationality =
      DeIdCardFront.DOCUMENT_NORMALIZED_TYPE + "." + "Nationality";
  static const String Birthplace =
      DeIdCardFront.DOCUMENT_NORMALIZED_TYPE + "." + "Birthplace";
  static const String ExpiryDate =
      DeIdCardFront.DOCUMENT_NORMALIZED_TYPE + "." + "ExpiryDate";
  static const String PIN =
      DeIdCardFront.DOCUMENT_NORMALIZED_TYPE + "." + "PIN";
  static const String Photo =
      DeIdCardFront.DOCUMENT_NORMALIZED_TYPE + "." + "Photo";
  static const String Signature =
      DeIdCardFront.DOCUMENT_NORMALIZED_TYPE + "." + "Signature";
  const DeIdCardFrontFieldsSignatures();
}

/// German ID card, back side
class DeIdCardBack extends GenericDocumentWrapper {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "DeIdCardBack";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE = "DeIdCardBack";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  /// Eye color
  late TextFieldWrapper eyeColor;

  /// Height
  late TextFieldWrapper height;

  /// Issue date
  late ValidatedTextFieldWrapper issueDate;

  /// Issuing authority
  late TextFieldWrapper issuingAuthority;

  /// Address
  late TextFieldWrapper address;

  /// Pseudonym
  late TextFieldWrapper pseudonym;

  /// Raw MRZ text value
  late TextFieldWrapper rawMRZ;

  /// Contains the MRZ sub-document.
  late MRZ mrz;

  DeIdCardBack(GenericDocument document) : super(document) {
    /// Eye color
    eyeColor = document
        .fieldByTypeName("EyeColor")!
        .let((field) => TextFieldWrapper(field));

    /// Height
    height = document
        .fieldByTypeName("Height")!
        .let((field) => TextFieldWrapper(field));

    /// Issue date
    issueDate = document
        .fieldByTypeName("IssueDate")!
        .let((field) => ValidatedTextFieldWrapper(field));

    /// Issuing authority
    issuingAuthority = document
        .fieldByTypeName("IssuingAuthority")!
        .let((field) => TextFieldWrapper(field));

    /// Address
    address = document
        .fieldByTypeName("Address")!
        .let((field) => TextFieldWrapper(field));

    /// Pseudonym
    pseudonym = document
        .fieldByTypeName("Pseudonym")!
        .let((field) => TextFieldWrapper(field));

    /// Raw MRZ text value
    rawMRZ = document
        .fieldByTypeName("RawMRZ")!
        .let((field) => TextFieldWrapper(field));
    mrz = document.childByDocumentType("MRZ")!.let((document) => MRZ(document));
  }
}

///  Class that contains all fields signatures of the DeIdCardBack document. Use it to Configure the document fields when running the scanner screen.
class DeIdCardBackFieldsSignatures {
  static const String EyeColor =
      DeIdCardBack.DOCUMENT_NORMALIZED_TYPE + "." + "EyeColor";
  static const String Height =
      DeIdCardBack.DOCUMENT_NORMALIZED_TYPE + "." + "Height";
  static const String IssueDate =
      DeIdCardBack.DOCUMENT_NORMALIZED_TYPE + "." + "IssueDate";
  static const String IssuingAuthority =
      DeIdCardBack.DOCUMENT_NORMALIZED_TYPE + "." + "IssuingAuthority";
  static const String Address =
      DeIdCardBack.DOCUMENT_NORMALIZED_TYPE + "." + "Address";
  static const String Pseudonym =
      DeIdCardBack.DOCUMENT_NORMALIZED_TYPE + "." + "Pseudonym";
  static const String RawMRZ =
      DeIdCardBack.DOCUMENT_NORMALIZED_TYPE + "." + "RawMRZ";
  const DeIdCardBackFieldsSignatures();
}

/// German travel passport (Reisepass)
class DePassport extends GenericDocumentWrapper {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "DePassport";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE = "DePassport";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  /// Document ID number (in the top-right corner)
  late TextFieldWrapper id;

  /// Surname
  late TextFieldWrapper surname;

  /// Maiden name
  late TextFieldWrapper? maidenName;

  /// Given names
  late TextFieldWrapper givenNames;

  /// Birth date
  late ValidatedTextFieldWrapper birthDate;

  /// Nationality
  late TextFieldWrapper nationality;

  /// Birthplace
  late TextFieldWrapper birthplace;

  /// Expiry date
  late ValidatedTextFieldWrapper expiryDate;

  /// Passport type
  late TextFieldWrapper passportType;

  /// Country code
  late TextFieldWrapper countryCode;

  /// Gender
  late TextFieldWrapper gender;

  /// Signature image
  late TextFieldWrapper signature;

  /// Photo image
  late TextFieldWrapper photo;

  /// Issue date
  late ValidatedTextFieldWrapper issueDate;

  /// Issuing authority
  late TextFieldWrapper issuingAuthority;

  /// Raw MRZ text value
  late TextFieldWrapper rawMRZ;

  /// Contains the MRZ sub-document.
  late MRZ mrz;

  DePassport(GenericDocument document) : super(document) {
    /// Document ID number (in the top-right corner)
    id =
        document.fieldByTypeName("ID")!.let((field) => TextFieldWrapper(field));

    /// Surname
    surname = document
        .fieldByTypeName("Surname")!
        .let((field) => TextFieldWrapper(field));

    /// Maiden name
    maidenName = document
        .fieldByTypeName("MaidenName")
        ?.let((field) => TextFieldWrapper(field));

    /// Given names
    givenNames = document
        .fieldByTypeName("GivenNames")!
        .let((field) => TextFieldWrapper(field));

    /// Birth date
    birthDate = document
        .fieldByTypeName("BirthDate")!
        .let((field) => ValidatedTextFieldWrapper(field));

    /// Nationality
    nationality = document
        .fieldByTypeName("Nationality")!
        .let((field) => TextFieldWrapper(field));

    /// Birthplace
    birthplace = document
        .fieldByTypeName("Birthplace")!
        .let((field) => TextFieldWrapper(field));

    /// Expiry date
    expiryDate = document
        .fieldByTypeName("ExpiryDate")!
        .let((field) => ValidatedTextFieldWrapper(field));

    /// Passport type
    passportType = document
        .fieldByTypeName("PassportType")!
        .let((field) => TextFieldWrapper(field));

    /// Country code
    countryCode = document
        .fieldByTypeName("CountryCode")!
        .let((field) => TextFieldWrapper(field));

    /// Gender
    gender = document
        .fieldByTypeName("Gender")!
        .let((field) => TextFieldWrapper(field));

    /// Signature image
    signature = document
        .fieldByTypeName("Signature")!
        .let((field) => TextFieldWrapper(field));

    /// Photo image
    photo = document
        .fieldByTypeName("Photo")!
        .let((field) => TextFieldWrapper(field));

    /// Issue date
    issueDate = document
        .fieldByTypeName("IssueDate")!
        .let((field) => ValidatedTextFieldWrapper(field));

    /// Issuing authority
    issuingAuthority = document
        .fieldByTypeName("IssuingAuthority")!
        .let((field) => TextFieldWrapper(field));

    /// Raw MRZ text value
    rawMRZ = document
        .fieldByTypeName("RawMRZ")!
        .let((field) => TextFieldWrapper(field));
    mrz = document.childByDocumentType("MRZ")!.let((document) => MRZ(document));
  }
}

///  Class that contains all fields signatures of the DePassport document. Use it to Configure the document fields when running the scanner screen.
class DePassportFieldsSignatures {
  static const String ID = DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "ID";
  static const String Surname =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "Surname";
  static const String MaidenName =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "MaidenName";
  static const String GivenNames =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "GivenNames";
  static const String BirthDate =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "BirthDate";
  static const String Nationality =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "Nationality";
  static const String Birthplace =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "Birthplace";
  static const String ExpiryDate =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "ExpiryDate";
  static const String PassportType =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "PassportType";
  static const String CountryCode =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "CountryCode";
  static const String Gender =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "Gender";
  static const String Signature =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "Signature";
  static const String Photo =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "Photo";
  static const String IssueDate =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "IssueDate";
  static const String IssuingAuthority =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "IssuingAuthority";
  static const String RawMRZ =
      DePassport.DOCUMENT_NORMALIZED_TYPE + "." + "RawMRZ";
  const DePassportFieldsSignatures();
}

/// German driver license (Führerschein), front side
class DeDriverLicenseFront extends GenericDocumentWrapper {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "DeDriverLicenseFront";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE = "DeDriverLicenseFront";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  /// Photo image
  late TextFieldWrapper photo;

  /// Surname
  late TextFieldWrapper surname;

  /// Given names
  late TextFieldWrapper givenNames;

  /// Birth date
  late ValidatedTextFieldWrapper birthDate;

  /// Birthplace
  late TextFieldWrapper birthplace;

  /// Issue date
  late ValidatedTextFieldWrapper issueDate;

  /// Issuing authority
  late TextFieldWrapper issuingAuthority;

  /// Expiry date
  late ValidatedTextFieldWrapper expiryDate;

  /// Document ID number (in the top-right corner)
  late TextFieldWrapper id;

  /// Signature image
  late TextFieldWrapper signature;

  /// Driver's license categories
  late TextFieldWrapper licenseCategories;

  DeDriverLicenseFront(GenericDocument document) : super(document) {
    /// Photo image
    photo = document
        .fieldByTypeName("Photo")!
        .let((field) => TextFieldWrapper(field));

    /// Surname
    surname = document
        .fieldByTypeName("Surname")!
        .let((field) => TextFieldWrapper(field));

    /// Given names
    givenNames = document
        .fieldByTypeName("GivenNames")!
        .let((field) => TextFieldWrapper(field));

    /// Birth date
    birthDate = document
        .fieldByTypeName("BirthDate")!
        .let((field) => ValidatedTextFieldWrapper(field));

    /// Birthplace
    birthplace = document
        .fieldByTypeName("Birthplace")!
        .let((field) => TextFieldWrapper(field));

    /// Issue date
    issueDate = document
        .fieldByTypeName("IssueDate")!
        .let((field) => ValidatedTextFieldWrapper(field));

    /// Issuing authority
    issuingAuthority = document
        .fieldByTypeName("IssuingAuthority")!
        .let((field) => TextFieldWrapper(field));

    /// Expiry date
    expiryDate = document
        .fieldByTypeName("ExpiryDate")!
        .let((field) => ValidatedTextFieldWrapper(field));

    /// Document ID number (in the top-right corner)
    id =
        document.fieldByTypeName("ID")!.let((field) => TextFieldWrapper(field));

    /// Signature image
    signature = document
        .fieldByTypeName("Signature")!
        .let((field) => TextFieldWrapper(field));

    /// Driver's license categories
    licenseCategories = document
        .fieldByTypeName("LicenseCategories")!
        .let((field) => TextFieldWrapper(field));
  }
}

///  Class that contains all fields signatures of the DeDriverLicenseFront document. Use it to Configure the document fields when running the scanner screen.
class DeDriverLicenseFrontFieldsSignatures {
  static const String Photo =
      DeDriverLicenseFront.DOCUMENT_NORMALIZED_TYPE + "." + "Photo";
  static const String Surname =
      DeDriverLicenseFront.DOCUMENT_NORMALIZED_TYPE + "." + "Surname";
  static const String GivenNames =
      DeDriverLicenseFront.DOCUMENT_NORMALIZED_TYPE + "." + "GivenNames";
  static const String BirthDate =
      DeDriverLicenseFront.DOCUMENT_NORMALIZED_TYPE + "." + "BirthDate";
  static const String Birthplace =
      DeDriverLicenseFront.DOCUMENT_NORMALIZED_TYPE + "." + "Birthplace";
  static const String IssueDate =
      DeDriverLicenseFront.DOCUMENT_NORMALIZED_TYPE + "." + "IssueDate";
  static const String IssuingAuthority =
      DeDriverLicenseFront.DOCUMENT_NORMALIZED_TYPE + "." + "IssuingAuthority";
  static const String ExpiryDate =
      DeDriverLicenseFront.DOCUMENT_NORMALIZED_TYPE + "." + "ExpiryDate";
  static const String ID =
      DeDriverLicenseFront.DOCUMENT_NORMALIZED_TYPE + "." + "ID";
  static const String Signature =
      DeDriverLicenseFront.DOCUMENT_NORMALIZED_TYPE + "." + "Signature";
  static const String LicenseCategories =
      DeDriverLicenseFront.DOCUMENT_NORMALIZED_TYPE + "." + "LicenseCategories";
  const DeDriverLicenseFrontFieldsSignatures();
}

/// German driver license (Führerschein), back side
class DeDriverLicenseBack extends GenericDocumentWrapper {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "DeDriverLicenseBack";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE = "DeDriverLicenseBack";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  /// Restrictions applied for the driver's license
  late TextFieldWrapper restrictions;

  /// Categories table row container
  late Categories categories;

  DeDriverLicenseBack(GenericDocument document) : super(document) {
    /// Restrictions applied for the driver's license
    restrictions = document
        .fieldByTypeName("Restrictions")!
        .let((field) => TextFieldWrapper(field));

    categories = document
        .childByDocumentType("Categories")!
        .let((document) => Categories(document));
  }
}

///  Class that contains all fields signatures of the DeDriverLicenseBack document. Use it to Configure the document fields when running the scanner screen.
class DeDriverLicenseBackFieldsSignatures {
  static const String Restrictions =
      DeDriverLicenseBack.DOCUMENT_NORMALIZED_TYPE + "." + "Restrictions";
  const DeDriverLicenseBackFieldsSignatures();
}

/// A category row from the categories table
abstract class Category extends GenericDocumentWrapper {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "Category";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE = "DeDriverLicenseBack.Category";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  /// Valid from
  late TextFieldWrapper validFrom;

  /// Valid until
  late TextFieldWrapper validUntil;

  /// Restrictions
  late TextFieldWrapper restrictions;

  Category(GenericDocument document) : super(document) {
    /// Valid from
    validFrom = document
        .fieldByTypeName("ValidFrom")!
        .let((field) => TextFieldWrapper(field));

    /// Valid until
    validUntil = document
        .fieldByTypeName("ValidUntil")!
        .let((field) => TextFieldWrapper(field));

    /// Restrictions
    restrictions = document
        .fieldByTypeName("Restrictions")!
        .let((field) => TextFieldWrapper(field));
  }
}

///  Class that contains all fields signatures of the Category document. Use it to Configure the document fields when running the scanner screen.
class CategoryFieldsSignatures {
  static const String ValidFrom =
      Category.DOCUMENT_NORMALIZED_TYPE + "." + "ValidFrom";
  static const String ValidUntil =
      Category.DOCUMENT_NORMALIZED_TYPE + "." + "ValidUntil";
  static const String Restrictions =
      Category.DOCUMENT_NORMALIZED_TYPE + "." + "Restrictions";
  const CategoryFieldsSignatures();
}

/// Categories table row container
class Categories extends GenericDocumentWrapper {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "Categories";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  /// Contains the A sub-document.
  late A a;

  /// Contains the A1 sub-document.
  late A1 a1;

  /// Contains the A2 sub-document.
  late A2 a2;

  /// Contains the B sub-document.
  late B b;

  /// Contains the B1 sub-document.
  late B1? b1;

  /// Contains the BE sub-document.
  late BE be;

  /// Contains the C sub-document.
  late C c;

  /// Contains the C1 sub-document.
  late C1 c1;

  /// Contains the C1E sub-document.
  late C1E c1e;

  /// Contains the CE sub-document.
  late CE ce;

  /// Contains the D sub-document.
  late D d;

  /// Contains the D1 sub-document.
  late D1 d1;

  /// Contains the D1E sub-document.
  late D1E d1e;

  /// Contains the DE sub-document.
  late DE de;

  /// Contains the L sub-document.
  late L l;

  /// Contains the M sub-document.
  late M m;

  /// Contains the T sub-document.
  late T t;

  Categories(GenericDocument document) : super(document) {
    a = document.childByDocumentType("A")!.let((document) => A(document));
    a1 = document.childByDocumentType("A1")!.let((document) => A1(document));
    a2 = document.childByDocumentType("A2")!.let((document) => A2(document));
    b = document.childByDocumentType("B")!.let((document) => B(document));
    b1 = document.childByDocumentType("B1")?.let((document) => B1(document));
    be = document.childByDocumentType("BE")!.let((document) => BE(document));
    c = document.childByDocumentType("C")!.let((document) => C(document));
    c1 = document.childByDocumentType("C1")!.let((document) => C1(document));
    c1e = document.childByDocumentType("C1E")!.let((document) => C1E(document));
    ce = document.childByDocumentType("CE")!.let((document) => CE(document));
    d = document.childByDocumentType("D")!.let((document) => D(document));
    d1 = document.childByDocumentType("D1")!.let((document) => D1(document));
    d1e = document.childByDocumentType("D1E")!.let((document) => D1E(document));
    de = document.childByDocumentType("DE")!.let((document) => DE(document));
    l = document.childByDocumentType("L")!.let((document) => L(document));
    m = document.childByDocumentType("M")!.let((document) => M(document));
    t = document.childByDocumentType("T")!.let((document) => T(document));
  }
}

///  Class that contains all fields signatures of the Categories document. Use it to Configure the document fields when running the scanner screen.
class CategoriesFieldsSignatures {
  const CategoriesFieldsSignatures();
}

/// The A document.
class A extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "A";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.A";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  A(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the A document. Use it to Configure the document fields when running the scanner screen.
class AFieldsSignatures {
  const AFieldsSignatures();
}

/// The A1 document.
class A1 extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "A1";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.A1";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  A1(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the A1 document. Use it to Configure the document fields when running the scanner screen.
class A1FieldsSignatures {
  const A1FieldsSignatures();
}

/// The A2 document.
class A2 extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "A2";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.A2";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  A2(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the A2 document. Use it to Configure the document fields when running the scanner screen.
class A2FieldsSignatures {
  const A2FieldsSignatures();
}

/// The B document.
class B extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "B";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.B";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  B(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the B document. Use it to Configure the document fields when running the scanner screen.
class BFieldsSignatures {
  const BFieldsSignatures();
}

/// The B1 document.
class B1 extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "B1";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.B1";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  B1(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the B1 document. Use it to Configure the document fields when running the scanner screen.
class B1FieldsSignatures {
  const B1FieldsSignatures();
}

/// The BE document.
class BE extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "BE";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.BE";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  BE(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the BE document. Use it to Configure the document fields when running the scanner screen.
class BEFieldsSignatures {
  const BEFieldsSignatures();
}

/// The C document.
class C extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "C";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.C";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  C(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the C document. Use it to Configure the document fields when running the scanner screen.
class CFieldsSignatures {
  const CFieldsSignatures();
}

/// The C1 document.
class C1 extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "C1";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.C1";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  C1(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the C1 document. Use it to Configure the document fields when running the scanner screen.
class C1FieldsSignatures {
  const C1FieldsSignatures();
}

/// The C1E document.
class C1E extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "C1E";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.C1E";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  C1E(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the C1E document. Use it to Configure the document fields when running the scanner screen.
class C1EFieldsSignatures {
  const C1EFieldsSignatures();
}

/// The CE document.
class CE extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "CE";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.CE";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  CE(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the CE document. Use it to Configure the document fields when running the scanner screen.
class CEFieldsSignatures {
  const CEFieldsSignatures();
}

/// The D document.
class D extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "D";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.D";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  D(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the D document. Use it to Configure the document fields when running the scanner screen.
class DFieldsSignatures {
  const DFieldsSignatures();
}

/// The D1 document.
class D1 extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "D1";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.D1";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  D1(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the D1 document. Use it to Configure the document fields when running the scanner screen.
class D1FieldsSignatures {
  const D1FieldsSignatures();
}

/// The D1E document.
class D1E extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "D1E";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.D1E";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  D1E(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the D1E document. Use it to Configure the document fields when running the scanner screen.
class D1EFieldsSignatures {
  const D1EFieldsSignatures();
}

/// The DE document.
class DE extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "DE";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.DE";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  DE(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the DE document. Use it to Configure the document fields when running the scanner screen.
class DEFieldsSignatures {
  const DEFieldsSignatures();
}

/// The L document.
class L extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "L";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.L";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  L(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the L document. Use it to Configure the document fields when running the scanner screen.
class LFieldsSignatures {
  const LFieldsSignatures();
}

/// The M document.
class M extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "M";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.M";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  M(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the M document. Use it to Configure the document fields when running the scanner screen.
class MFieldsSignatures {
  const MFieldsSignatures();
}

/// The T document.
class T extends Category {
  /// Generic document type name of this document wrapper.
  static const String DOCUMENT_TYPE = "T";

  /// Normalized document type name of this document wrapper.
  static const String DOCUMENT_NORMALIZED_TYPE =
      "DeDriverLicenseBack.Categories.T";

  @override
  DocumentType get documentType => DOCUMENT_NORMALIZED_TYPE.toDocumentType();

  T(GenericDocument document) : super(document);
}

///  Class that contains all fields signatures of the T document. Use it to Configure the document fields when running the scanner screen.
class TFieldsSignatures {
  const TFieldsSignatures();
}

GenericDocumentWrapper wrapToDocumentType(GenericDocument document) {
  switch (document.type?.name) {
    case MRZ.DOCUMENT_TYPE:
      return MRZ(document);

    case DeIdCardFront.DOCUMENT_TYPE:
      return DeIdCardFront(document);

    case DeIdCardBack.DOCUMENT_TYPE:
      return DeIdCardBack(document);

    case DePassport.DOCUMENT_TYPE:
      return DePassport(document);

    case DeDriverLicenseFront.DOCUMENT_TYPE:
      return DeDriverLicenseFront(document);

    case DeDriverLicenseBack.DOCUMENT_TYPE:
      return DeDriverLicenseBack(document);
    default:
      return GenericDocumentWrapper(document);
  }
}

enum DocumentType {
  Unknown,
  MRZ,
  DeIdCardFront,
  DeIdCardBack,
  DePassport,
  DeDriverLicenseFront,
  DeDriverLicenseBack,

  DeDriverLicenseBack_categories;
}

extension DocumentTypeExtention on String {
  DocumentType toDocumentType() {
    late DocumentType type;
    switch (this) {
      case MRZ.DOCUMENT_NORMALIZED_TYPE:
        type = DocumentType.MRZ;
        break;
      case DeIdCardFront.DOCUMENT_NORMALIZED_TYPE:
        type = DocumentType.DeIdCardFront;
        break;
      case DeIdCardBack.DOCUMENT_NORMALIZED_TYPE:
        type = DocumentType.DeIdCardBack;
        break;
      case DePassport.DOCUMENT_NORMALIZED_TYPE:
        type = DocumentType.DePassport;
        break;
      case DeDriverLicenseFront.DOCUMENT_NORMALIZED_TYPE:
        type = DocumentType.DeDriverLicenseFront;
        break;
      case DeDriverLicenseBack.DOCUMENT_NORMALIZED_TYPE:
        type = DocumentType.DeDriverLicenseBack;
        break;
      case Categories.DOCUMENT_NORMALIZED_TYPE:
        type = DocumentType.DeDriverLicenseBack_categories;
        break;
    }
    return type;
  }
}
