import 'package:json_annotation/json_annotation.dart';

part 'mc_scanning_data.g.dart';

@JsonSerializable(
    createFactory: true, includeIfNull: false, createToJson: false)
class MedicalCertificateResult {
  /// Defines a result of the Medical Certificate scanning flow. */ class MedicalCertificateRecognizerResult {

  /// Defines whether recognition was successful
  bool recognitionSuccessful;

  /// Vector of found checkboxes

  List<MedicalCertificateCheckBox> checkboxes;

  /// Vector of validated dates

  List<DateRecord> dates;

  MedicalCertificatePatientInfoBox patientInfoBox;

  /// Type of MC form

  McFormType mcFormType = McFormType.Unknown;

  Uri? croppedDocumentURI;

  MedicalCertificateResult(this.recognitionSuccessful, this.checkboxes,
      this.dates, this.patientInfoBox, this.mcFormType);

  factory MedicalCertificateResult.fromJson(Map<String, dynamic> json) =>
      _$MedicalCertificateResultFromJson(json);
}

/// Structure to contain full information about found box
@JsonSerializable(
    createFactory: true, includeIfNull: false, createToJson: false)
class MedicalCertificatePatientInfoBox {
  /// Information about the patient
  List<MedicalCertificatePatientInfoField> fields;

  /// If box is a checkbox, this field shows whether the checkbox is checked. Not used otherwise
  bool hasContents;

  MedicalCertificatePatientInfoBox(this.fields, this.hasContents);

  factory MedicalCertificatePatientInfoBox.fromJson(
          Map<String, dynamic> json) =>
      _$MedicalCertificatePatientInfoBoxFromJson(json);
}

/// Structure to contain full information about found box
@JsonSerializable(
    createFactory: true, includeIfNull: false, createToJson: false)
class MedicalCertificateCheckBox {
  /// Box subtype

  MedicalCertificateCheckBoxType type;

  /// If box is a checkbox, this field shows whether the checkbox is checked. Not used otherwise

  bool hasContents;

  /// Contents validation confidence value

  double contentsValidationConfidenceValue;

  MedicalCertificateCheckBox(
      this.type, this.hasContents, this.contentsValidationConfidenceValue);

  factory MedicalCertificateCheckBox.fromJson(Map<String, dynamic> json) =>
      _$MedicalCertificateCheckBoxFromJson(json);
}

/// Box detailed subtype
enum MedicalCertificateCheckBoxType {
  McBoxUnknown,
  McBoxPatientInfo,
  McBoxWorkAccident,
  McBoxAssignedToAccidentInsuranceDoctor,
  McBoxInitialCertificate,
  McBoxRenewedCertificate,
  McBoxInsuredPayCase,
  McBoxFinalCertificate,
  McBoxRequiresCareYes,
  McBoxRequiresCareNo,
  McBoxAccidentYes,
  McBoxAccidentNo,
  McBoxOtherAccident,
  McBoxEntitlementToContinuedPaymentYes,
  McBoxEntitlementToContinuedPaymentNo,
  McBoxSickPayWasClaimedNo,
  McBoxSickPayWasClaimedYes,
  McBoxSingleParentNo,
  McBoxSingleParentYes
}

/// Structure to contain date record information
@JsonSerializable(
    createFactory: true, includeIfNull: false, createToJson: false)
class DateRecord {
  /// Date string
  String dateString;

  /// Date type
  DateRecordType type;

  /// Recognition confidence value
  double recognitionConfidenceValue;

  DateRecord(this.dateString, this.type, this.recognitionConfidenceValue);

  factory DateRecord.fromJson(Map<String, dynamic> json) =>
      _$DateRecordFromJson(json);
}

/// Type of a date record
enum DateRecordType {
  DateRecordIncapableOfWorkSince,
  DateRecordIncapableOfWorkUntil,
  DateRecordDiagnosedOn,
  DateRecordDocumentDate,
  DateRecordBirthDate,
  DateRecordChildNeedsCareFrom,
  DateRecordChildNeedsCareUntil,
  DateRecordUndefined
}

/// Represents an item of the patient info
@JsonSerializable(
    createFactory: true, includeIfNull: false, createToJson: false)
class MedicalCertificatePatientInfoField {
  MedicalCertificatePatientInfoFieldType patientInfoFieldType;
  String value;
  double confidenceValue;
  MedicalCertificatePatientInfoField(
      this.patientInfoFieldType, this.confidenceValue, this.value);

  factory MedicalCertificatePatientInfoField.fromJson(
          Map<String, dynamic> json) =>
      _$MedicalCertificatePatientInfoFieldFromJson(json);
}

/// Form type of the Medical certificate
enum McFormType {
  Form_1a,
  Form_1b,
  Form_1c,
  Form_1d,
  Form_21a,
  Form_21a_back,
  Form_1b_Custom,
  Unknown
}

/// Type of the item of the patient info field
enum MedicalCertificatePatientInfoFieldType {
  InsuranceProvider,
  FirstName,
  LastName,
  AddressString1,
  AddressString2,
  Diagnose,
  HealthInsuranceNumber,
  InsuredPersonNumber,
  Status,
  PlaceOfOperationNumber,
  DoctorNumber,
  Undefined
}
