import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';
import 'dart:ui';

import 'package:flutter/painting.dart';

import 'json/common.dart';
import 'json/common_platform.dart';

String? toColorString(Color? color) {
  if (Platform.isIOS) {
    return toRgbaString(color);
  }
  return toArgbString(color);
}

String? toRgbaString(Color? color) {
  return color != null
      ? '#${color.red.toRadixString(16).padLeft(2, '0')}${color.green.toRadixString(16).padLeft(2, '0')}${color.blue.toRadixString(16).padLeft(2, '0')}${color.alpha.toRadixString(16).padLeft(2, '0')}'
      : null;
}

String? toArgbString(Color? color) {
  return color != null
      ? '#${color.value.toRadixString(16).padLeft(8, '0')}'
      : null;
}

Color? colorFromString(String? color) {
  if (color != null) {
    var valueString = color.replaceAll('#', '');
    var value = int.parse(valueString, radix: 16);
    return Color(value);
  }
  return null;
}

Uint8List uintListFromString(String string) {
  return const Base64Decoder().convert(string);
}

Uri? stringToFileUri(String? string) {
  if (string != null) {
    return Uri.parse(string);
  }
  return null;
}

String? fileUriToString(Uri? uri) {
  return uri?.toString();
}

DateTime? dateFromString(String? string) {
  if (string != null) {
    return DateTime.parse(string);
  }
  return null;
}

String imageFilterJsonValue(ImageFilterType filter) {
  return filter.toString().replaceFirst('ImageFilterType.', '');
}

const imageFormatMap = {ImageFormat.JPG: 'JPG', ImageFormat.PNG: 'PNG'};
const contourDetectorTypeMap = {
  DocumentDetectorMode.ML_BASED: 'ML_BASED',
  DocumentDetectorMode.EDGE_BASED: 'EDGE_BASED'
};

const allDocumentFormatTypes = [
  BarcodeDocumentFormat.AAMVA,
  BarcodeDocumentFormat.BOARDING_PASS,
  BarcodeDocumentFormat.DE_MEDICAL_PLAN,
  BarcodeDocumentFormat.MEDICAL_CERTIFICATE,
  BarcodeDocumentFormat.ID_CARD_PDF_417,
  BarcodeDocumentFormat.SEPA,
  BarcodeDocumentFormat.SWISS_QR,
  BarcodeDocumentFormat.VCARD,
  BarcodeDocumentFormat.GS1
];

extension KotlinFunctionsForObject<T extends Object> on T {
  R let<R>(R Function(T self) operationFor) {
    return operationFor(this);
  }

  T also(void Function(T self) operationFor) {
    operationFor(this);
    return this;
  }
}
